@@ -36,9 +36,6 @@ import android.graphics.Bitmap;
 import android.graphics.BitmapFactory;
 import android.media.AudioManager;
 import android.media.AudioManager.OnAudioFocusChangeListener;
-import android.media.MediaMetadataRetriever;
-import android.media.RemoteControlClient;
-import android.media.RemoteControlClient.MetadataEditor;
 import android.net.Uri;
 import android.os.Binder;
 import android.os.Build;
@@ -50,12 +47,15 @@ import android.os.PowerManager;
 import android.preference.PreferenceManager;
 import android.support.annotation.MainThread;
 import android.support.annotation.Nullable;
-import android.support.v4.app.NotificationCompat;
 import android.support.v4.app.NotificationManagerCompat;
+import android.support.v4.media.MediaMetadataCompat;
+import android.support.v4.media.session.MediaSessionCompat;
+import android.support.v4.media.session.PlaybackStateCompat;
+import android.support.v7.app.NotificationCompat;
+import android.telephony.PhoneStateListener;
 import android.telephony.TelephonyManager;
+import android.text.TextUtils;
 import android.util.Log;
-import android.view.View;
-import android.widget.RemoteViews;
 import android.widget.Toast;
 
 import org.videolan.libvlc.IVLCVout;
@@ -65,18 +65,26 @@ import org.videolan.libvlc.MediaList;
 import org.videolan.libvlc.MediaPlayer;
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.gui.AudioPlayerContainerActivity;
-import org.videolan.vlc.gui.audio.AudioUtil;
+import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
+import org.videolan.vlc.media.MediaDatabase;
+import org.videolan.vlc.media.MediaUtils;
+import org.videolan.vlc.media.MediaWrapper;
+import org.videolan.vlc.media.MediaWrapperList;
+import org.videolan.vlc.util.FileUtils;
+import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.VLCOptions;
 import org.videolan.vlc.util.WeakHandler;
+import org.videolan.vlc.widget.VLCAppWidgetProvider;
 
 import java.io.File;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Calendar;
+import java.util.Collections;
 import java.util.List;
 import java.util.Locale;
 import java.util.Random;
@@ -89,19 +97,15 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     private static final int SHOW_PROGRESS = 0;
     private static final int SHOW_TOAST = 1;
-    public static final String ACTION_REMOTE_GENERIC = "org.videolan.vlc.remote.";
-    public static final String ACTION_REMOTE_BACKWARD = "org.videolan.vlc.remote.Backward";
-    public static final String ACTION_REMOTE_PLAY = "org.videolan.vlc.remote.Play";
-    public static final String ACTION_REMOTE_PLAYPAUSE = "org.videolan.vlc.remote.PlayPause";
-    public static final String ACTION_REMOTE_PAUSE = "org.videolan.vlc.remote.Pause";
-    public static final String ACTION_REMOTE_STOP = "org.videolan.vlc.remote.Stop";
-    public static final String ACTION_REMOTE_FORWARD = "org.videolan.vlc.remote.Forward";
-    public static final String ACTION_REMOTE_LAST_PLAYLIST = "org.videolan.vlc.remote.LastPlaylist";
-    public static final String ACTION_REMOTE_RESUME_VIDEO = "org.videolan.vlc.remote.ResumeVideo";
-    public static final String ACTION_WIDGET_INIT = "org.videolan.vlc.widget.INIT";
-    public static final String ACTION_WIDGET_UPDATE = "org.videolan.vlc.widget.UPDATE";
-    public static final String ACTION_WIDGET_UPDATE_COVER = "org.videolan.vlc.widget.UPDATE_COVER";
-    public static final String ACTION_WIDGET_UPDATE_POSITION = "org.videolan.vlc.widget.UPDATE_POSITION";
+    public static final String ACTION_REMOTE_GENERIC =  Strings.buildPkgString("remote.");
+    public static final String ACTION_REMOTE_BACKWARD = ACTION_REMOTE_GENERIC+"Backward";
+    public static final String ACTION_REMOTE_PLAY = ACTION_REMOTE_GENERIC+"Play";
+    public static final String ACTION_REMOTE_PLAYPAUSE = ACTION_REMOTE_GENERIC+"PlayPause";
+    public static final String ACTION_REMOTE_PAUSE = ACTION_REMOTE_GENERIC+"Pause";
+    public static final String ACTION_REMOTE_STOP = ACTION_REMOTE_GENERIC+"Stop";
+    public static final String ACTION_REMOTE_FORWARD = ACTION_REMOTE_GENERIC+"Forward";
+    public static final String ACTION_REMOTE_LAST_PLAYLIST = ACTION_REMOTE_GENERIC+"LastPlaylist";
+    public static final String ACTION_REMOTE_RESUME_VIDEO = ACTION_REMOTE_GENERIC+"ResumeVideo";
 
     public interface Callback {
         void update();
@@ -123,6 +127,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     private final IBinder mBinder = new LocalBinder();
     private MediaWrapperList mMediaList = new MediaWrapperList();
     private MediaPlayer mMediaPlayer;
+    private boolean mParsed = false;
     private boolean mSeekable = false;
     private boolean mPausable = false;
     private boolean mIsAudioTrack = false;
@@ -147,6 +152,12 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     // Playback management
 
+    MediaSessionCompat mMediaSession;
+    protected MediaSessionCallback mSessionCallback;
+    private static final long PLAYBACK_ACTIONS = PlaybackStateCompat.ACTION_PAUSE
+            | PlaybackStateCompat.ACTION_PLAY | PlaybackStateCompat.ACTION_STOP
+            | PlaybackStateCompat.ACTION_SKIP_TO_NEXT | PlaybackStateCompat.ACTION_SKIP_TO_PREVIOUS;
+
     public static final int REPEAT_NONE = 0;
     public static final int REPEAT_ONE = 1;
     public static final int REPEAT_ALL = 2;
@@ -159,7 +170,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     /**
      * RemoteControlClient is for lock screen playback control.
      */
-    private RemoteControlClient mRemoteControlClient = null;
     private RemoteControlClientReceiver mRemoteControlClientReceiver = null;
     /**
      * Last widget position update timestamp
@@ -226,14 +236,10 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         filter.addAction(ACTION_REMOTE_FORWARD);
         filter.addAction(ACTION_REMOTE_LAST_PLAYLIST);
         filter.addAction(ACTION_REMOTE_RESUME_VIDEO);
-        filter.addAction(ACTION_WIDGET_INIT);
+        filter.addAction(VLCAppWidgetProvider.ACTION_WIDGET_INIT);
         filter.addAction(Intent.ACTION_HEADSET_PLUG);
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
         filter.addAction(VLCApplication.SLEEP_INTENT);
-        if (readPhoneState()) {
-            filter.addAction(VLCApplication.INCOMING_CALL_INTENT);
-            filter.addAction(VLCApplication.CALL_ENDED_INTENT);
-        }
         registerReceiver(mReceiver, filter);
         registerV21();
 
@@ -255,29 +261,11 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         } catch (PackageManager.NameNotFoundException e) {
             mPebbleEnabled = false;
         }
-    }
 
-    /**
-     * A function to control the Remote Control Client. It is needed for
-     * compatibility with devices below Ice Cream Sandwich (4.0).
-     *
-     * @param state Playback state
-     */
-    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
-    private void setRemoteControlClientPlaybackState(int state) {
-        if (!AndroidUtil.isICSOrLater() || mRemoteControlClient == null)
-            return;
-
-        switch (state) {
-            case MediaPlayer.Event.Playing:
-                mRemoteControlClient.setPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);
-                break;
-            case MediaPlayer.Event.Paused:
-                mRemoteControlClient.setPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
-                break;
-            case MediaPlayer.Event.Stopped:
-                mRemoteControlClient.setPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
-                break;
+        if (readPhoneState()) {
+            initPhoneListener();
+            TelephonyManager tm = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
+            tm.listen(mPhoneStateListener, mPhoneEvents);
         }
     }
 
@@ -314,14 +302,25 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             mRemoteControlClientReceiver = null;
         }
         mMediaPlayer.release();
-    }
 
+        if (readPhoneState()) {
+            TelephonyManager tm = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
+            tm.listen(mPhoneStateListener, PhoneStateListener.LISTEN_NONE);
+        }
+    }
 
     @Override
     public IBinder onBind(Intent intent) {
         return mBinder;
     }
 
+    @Override
+    public boolean onUnbind(Intent intent) {
+        if (!isPlaying())
+            stopSelf();
+        return true;
+    }
+
     public IVLCVout getVLCVout()  {
         return mMediaPlayer.getVLCVout();
     }
@@ -380,32 +379,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         };
     }
 
-    /**
-     * Set up the remote control and tell the system we want to be the default receiver for the MEDIA buttons
-     */
-    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
-    public void changeRemoteControlClient(AudioManager am, boolean acquire) {
-        if (acquire) {
-            Intent mediaButtonIntent = new Intent(Intent.ACTION_MEDIA_BUTTON);
-            mediaButtonIntent.setComponent(mRemoteControlClientReceiverComponent);
-            PendingIntent mediaPendingIntent = PendingIntent.getBroadcast(this, 0, mediaButtonIntent, 0);
-
-            // create and register the remote control client
-            mRemoteControlClient = new RemoteControlClient(mediaPendingIntent);
-            am.registerRemoteControlClient(mRemoteControlClient);
-
-            mRemoteControlClient.setTransportControlFlags(
-                    RemoteControlClient.FLAG_KEY_MEDIA_PLAY |
-                            RemoteControlClient.FLAG_KEY_MEDIA_PAUSE |
-                            RemoteControlClient.FLAG_KEY_MEDIA_PREVIOUS |
-                            RemoteControlClient.FLAG_KEY_MEDIA_NEXT |
-                            RemoteControlClient.FLAG_KEY_MEDIA_STOP);
-        } else {
-            am.unregisterRemoteControlClient(mRemoteControlClient);
-            mRemoteControlClient = null;
-        }
-    }
-
     @TargetApi(Build.VERSION_CODES.FROYO)
     private void changeAudioFocusFroyoOrLater(boolean acquire) {
         final AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
@@ -418,9 +391,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                         AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);
                 if (result == AudioManager.AUDIOFOCUS_REQUEST_GRANTED) {
                     am.setParameters("bgm_state=true");
-                    am.registerMediaButtonEventReceiver(mRemoteControlClientReceiverComponent);
-                    if (AndroidUtil.isICSOrLater())
-                        changeRemoteControlClient(am, acquire);
                     mHasAudioFocus = true;
                 }
             }
@@ -428,9 +398,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             if (mHasAudioFocus) {
                 final int result = am.abandonAudioFocus(mAudioFocusListener);
                 am.setParameters("bgm_state=false");
-                am.unregisterMediaButtonEventReceiver(mRemoteControlClientReceiverComponent);
-                if (AndroidUtil.isICSOrLater())
-                    changeRemoteControlClient(am, acquire);
                 mHasAudioFocus = false;
             }
         }
@@ -474,25 +441,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 return;
             }
 
-            if (readPhoneState()) {
-                /*
-                 * Incoming Call : Pause if VLC is playing audio or video.
-                 */
-                if (action.equalsIgnoreCase(VLCApplication.INCOMING_CALL_INTENT)) {
-                    mWasPlayingAudio = mMediaPlayer.isPlaying() && hasCurrentMedia();
-                    if (mWasPlayingAudio)
-                        pause();
-                }
-
-                /*
-                 * Call ended : Play only if VLC was playing audio.
-                 */
-                if (action.equalsIgnoreCase(VLCApplication.CALL_ENDED_INTENT)
-                        && mWasPlayingAudio) {
-                    play();
-                }
-            }
-
             // skip all headsets events if there is a call
             TelephonyManager telManager = (TelephonyManager) VLCApplication.getAppContext().getSystemService(Context.TELEPHONY_SERVICE);
             if (telManager != null && telManager.getCallState() != TelephonyManager.CALL_STATE_IDLE)
@@ -529,7 +477,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 loadLastPlaylist();
             } else if (action.equalsIgnoreCase(ACTION_REMOTE_RESUME_VIDEO)) {
                 switchToVideo();
-            } else if (action.equalsIgnoreCase(ACTION_WIDGET_INIT)) {
+            } else if (action.equalsIgnoreCase(VLCAppWidgetProvider.ACTION_WIDGET_INIT)) {
                 updateWidget();
             }
 
@@ -577,17 +525,21 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         @Override
         public void onEvent(Media.Event event) {
             switch (event.type) {
+                case Media.Event.MetaChanged:
+                    /* Update Meta if file is already parsed */
+                    if (!mParsed)
+                        break;
+                    Log.i(TAG, "Media.Event.MetaChanged: " + event.getMetaId());
+
                 case Media.Event.ParsedChanged:
                     Log.i(TAG, "Media.Event.ParsedChanged");
                     final MediaWrapper mw = getCurrentMedia();
                     if (mw != null)
                         mw.updateMeta(mMediaPlayer);
                     executeUpdate();
-                    showNotification();
-                    updateRemoteControlClientMetadata();
-                    break;
-                case Media.Event.MetaChanged:
+                    mParsed = true;
                     break;
+
             }
             for (Callback callback : mCallbacks)
                 callback.onMediaEvent(event);
@@ -599,8 +551,10 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         public void onEvent(MediaPlayer.Event event) {
             switch (event.type) {
                 case MediaPlayer.Event.Playing:
+
                     Log.i(TAG, "MediaPlayer.Event.Playing");
                     executeUpdate();
+                    publishState(event.type);
                     executeUpdateProgress();
 
                     final MediaWrapper mw = mMediaList.getMedia(mCurrentIndex);
@@ -622,7 +576,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     }
 
                     changeAudioFocus(true);
-                    setRemoteControlClientPlaybackState(event.type);
                     showNotification();
                     if (!mWakeLock.isHeld())
                         mWakeLock.acquire();
@@ -630,17 +583,17 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 case MediaPlayer.Event.Paused:
                     Log.i(TAG, "MediaPlayer.Event.Paused");
                     executeUpdate();
+                    publishState(event.type);
                     executeUpdateProgress();
                     showNotification();
-                    setRemoteControlClientPlaybackState(event.type);
                     if (mWakeLock.isHeld())
                         mWakeLock.release();
                     break;
                 case MediaPlayer.Event.Stopped:
                     Log.i(TAG, "MediaPlayer.Event.Stopped");
                     executeUpdate();
+                    publishState(event.type);
                     executeUpdateProgress();
-                    setRemoteControlClientPlaybackState(event.type);
                     if (mWakeLock.isHeld())
                         mWakeLock.release();
                     changeAudioFocus(false);
@@ -676,6 +629,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     if (event.getEsChangedType() == Media.Track.Type.Video) {
                         if (!handleVout()) {
                             /* Update notification content intent: resume video or resume audio activity */
+                            updateMetadata();
                             showNotification();
                         }
                     }
@@ -783,6 +737,8 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         }
         if (updateWidget)
             updateWidget();
+        updateMetadata();
+
     }
 
     private void executeUpdateProgress() {
@@ -845,27 +801,26 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if (mMediaPlayer.getVLCVout().areViewsAttached())
             return;
         try {
-            MediaWrapper media = getCurrentMedia();
-            if (media == null)
-                return;
-            Bitmap cover = AudioUtil.getCover(this, media, 64);
-            String title = media.getTitle();
-            String artist = Util.getMediaArtist(this, media);
-            String album = Util.getMediaAlbum(this, media);
+            MediaMetadataCompat metaData = mMediaSession.getController().getMetadata();
+            String title = metaData.getString(MediaMetadataCompat.METADATA_KEY_TITLE);
+            String artist = metaData.getString(MediaMetadataCompat.METADATA_KEY_ALBUM_ARTIST);
+            String album = metaData.getString(MediaMetadataCompat.METADATA_KEY_ALBUM);
+            Bitmap cover = metaData.getBitmap(MediaMetadataCompat.METADATA_KEY_ALBUM_ART);
+            if (cover == null)
+                cover = BitmapFactory.decodeResource(VLCApplication.getAppContext().getResources(), R.drawable.icon);
             Notification notification;
 
-            if (media.isArtistUnknown() && media.isAlbumUnknown() && media.getNowPlaying() != null) {
-                artist = media.getNowPlaying();
-                album = "";
-            }
-
             //Watch notification dismissed
             PendingIntent piStop = PendingIntent.getBroadcast(this, 0,
                     new Intent(ACTION_REMOTE_STOP), PendingIntent.FLAG_UPDATE_CURRENT);
 
             // add notification to status bar
-            NotificationCompat.Builder builder = new NotificationCompat.Builder(this)
-                .setSmallIcon(R.drawable.ic_stat_vlc)
+            NotificationCompat.Builder builder = new NotificationCompat.Builder(this);
+            builder.setSmallIcon(R.drawable.ic_stat_vlc)
+                .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
+                .setContentTitle(title)
+                .setContentText(artist + " - " + album)
+                .setLargeIcon(cover)
                 .setTicker(title + " - " + artist)
                 .setAutoCancel(!mMediaPlayer.isPlaying())
                 .setOngoing(mMediaPlayer.isPlaying())
@@ -886,60 +841,27 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
             }
 
-            if (AndroidUtil.isJellyBeanOrLater()) {
-                Intent iBackward = new Intent(ACTION_REMOTE_BACKWARD);
-                Intent iPlay = new Intent(ACTION_REMOTE_PLAYPAUSE);
-                Intent iForward = new Intent(ACTION_REMOTE_FORWARD);
-                PendingIntent piBackward = PendingIntent.getBroadcast(this, 0, iBackward, PendingIntent.FLAG_UPDATE_CURRENT);
-                PendingIntent piPlay = PendingIntent.getBroadcast(this, 0, iPlay, PendingIntent.FLAG_UPDATE_CURRENT);
-                PendingIntent piForward = PendingIntent.getBroadcast(this, 0, iForward, PendingIntent.FLAG_UPDATE_CURRENT);
+            builder.setContentIntent(pendingIntent);
 
-                RemoteViews view = new RemoteViews(BuildConfig.APPLICATION_ID, R.layout.notification);
-                view.setImageViewBitmap(R.id.cover, cover == null ? BitmapFactory.decodeResource(getResources(), R.drawable.icon) : cover);
-                view.setTextViewText(R.id.songName, title);
-                view.setTextViewText(R.id.artist, artist);
-                view.setImageViewResource(R.id.play_pause, mMediaPlayer.isPlaying() ? R.drawable.ic_pause_w : R.drawable.ic_play_w);
-                view.setOnClickPendingIntent(R.id.play_pause, piPlay);
-                view.setOnClickPendingIntent(R.id.forward, piForward);
-                view.setViewVisibility(R.id.forward, hasNext() ? View.VISIBLE : View.INVISIBLE);
-                view.setOnClickPendingIntent(R.id.stop, piStop);
-                view.setOnClickPendingIntent(R.id.content, pendingIntent);
+            PendingIntent piBackward = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_REMOTE_BACKWARD), PendingIntent.FLAG_UPDATE_CURRENT);
+            PendingIntent piPlay = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_REMOTE_PLAYPAUSE), PendingIntent.FLAG_UPDATE_CURRENT);
+            PendingIntent piForward = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_REMOTE_FORWARD), PendingIntent.FLAG_UPDATE_CURRENT);
 
-                RemoteViews view_expanded = new RemoteViews(BuildConfig.APPLICATION_ID, R.layout.notification_expanded);
-                view_expanded.setImageViewBitmap(R.id.cover, cover == null ? BitmapFactory.decodeResource(getResources(), R.drawable.icon) : cover);
-                view_expanded.setTextViewText(R.id.songName, title);
-                view_expanded.setTextViewText(R.id.artist, artist);
-                view_expanded.setTextViewText(R.id.album, album);
-                view_expanded.setImageViewResource(R.id.play_pause, mMediaPlayer.isPlaying() ? R.drawable.ic_pause_w : R.drawable.ic_play_w);
-                view_expanded.setOnClickPendingIntent(R.id.backward, piBackward);
-                view_expanded.setViewVisibility(R.id.backward, hasPrevious() ? View.VISIBLE : View.INVISIBLE);
-                view_expanded.setOnClickPendingIntent(R.id.play_pause, piPlay);
-                view_expanded.setOnClickPendingIntent(R.id.forward, piForward);
-                view_expanded.setViewVisibility(R.id.forward, hasNext() ? View.VISIBLE : View.INVISIBLE);
-                view_expanded.setOnClickPendingIntent(R.id.stop, piStop);
-                view_expanded.setOnClickPendingIntent(R.id.content, pendingIntent);
+            builder.addAction(R.drawable.ic_previous_w, getString(R.string.previous), piBackward);
+            if (mMediaPlayer.isPlaying())
+                builder.addAction(R.drawable.ic_pause_w, getString(R.string.pause), piPlay);
+            else
+                builder.addAction(R.drawable.ic_play_w, getString(R.string.play), piPlay);
+            builder.addAction(R.drawable.ic_next_w, getString(R.string.next), piForward);
 
-                if (AndroidUtil.isLolliPopOrLater()){
-                    //Hide stop button on pause, we swipe notification to stop
-                    view.setViewVisibility(R.id.stop, mMediaPlayer.isPlaying() ? View.VISIBLE : View.INVISIBLE);
-                    view_expanded.setViewVisibility(R.id.stop, mMediaPlayer.isPlaying() ? View.VISIBLE : View.INVISIBLE);
-                    //Make notification appear on lockscreen
-                    builder.setVisibility(Notification.VISIBILITY_PUBLIC);
-                }
+            builder.setStyle(new NotificationCompat.MediaStyle()
+                            .setMediaSession(mMediaSession.getSessionToken())
+                            .setShowActionsInCompactView(new int[] {0,1,2})
+                            .setShowCancelButton(true)
+                            .setCancelButtonIntent(piStop)
+            );
 
-                notification = builder.build();
-                notification.contentView = view;
-                notification.bigContentView = view_expanded;
-            }
-            else {
-                builder.setLargeIcon(cover == null ? BitmapFactory.decodeResource(getResources(), R.drawable.icon) : cover)
-                       .setContentTitle(title)
-                        .setContentText(AndroidUtil.isJellyBeanOrLater() ? artist
-                                        : Util.getMediaSubtitle(this, media))
-                       .setContentInfo(album)
-                       .setContentIntent(pendingIntent);
-                notification = builder.build();
-            }
+            notification = builder.build();
 
             startService(new Intent(this, PlaybackService.class));
             if (!AndroidUtil.isLolliPopOrLater() || mMediaPlayer.isPlaying())
@@ -987,7 +909,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if(hasCurrentMedia()) {
             mMediaPlayer.play();
             mHandler.sendEmptyMessage(SHOW_PROGRESS);
-            showNotification();
+            updateMetadata();
             updateWidget();
             broadcastMetadata();
         }
@@ -995,6 +917,11 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     @MainThread
     public void stop() {
+        if (mMediaSession != null) {
+            mMediaSession.setActive(false);
+            mMediaSession.release();
+            mMediaSession = null;
+        }
         if (mMediaPlayer == null)
             return;
         savePosition();
@@ -1007,7 +934,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             media.release();
         }
         mMediaList.removeEventListener(mListEventListener);
-        setRemoteControlClientPlaybackState(MediaPlayer.Event.Stopped);
         mCurrentIndex = -1;
         mPrevious.clear();
         mHandler.removeMessages(SHOW_PROGRESS);
@@ -1016,6 +942,8 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         executeUpdate();
         executeUpdateProgress();
         changeAudioFocus(false);
+
+        stopSelf();
     }
 
     private void determinePrevAndNextIndices() {
@@ -1081,51 +1009,108 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         }
     }
 
-    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
-    private void updateRemoteControlClientMetadata() {
-        if (!AndroidUtil.isICSOrLater()) // NOP check
-            return;
+    private void initMediaSession() {
+        mSessionCallback = new MediaSessionCallback();
+        mMediaSession = new MediaSessionCompat(this, "VLC");
+        mMediaSession.setFlags(MediaSessionCompat.FLAG_HANDLES_MEDIA_BUTTONS
+                | MediaSessionCompat.FLAG_HANDLES_TRANSPORT_CONTROLS);
+        mMediaSession.setCallback(mSessionCallback);
+        try {
+            mMediaSession.setActive(true);
+        } catch (NullPointerException e) {
+            // Some versions of KitKat do not support AudioManager.registerMediaButtonIntent
+            // with a PendingIntent. They will throw a NullPointerException, in which case
+            // they should be able to activate a MediaSessionCompat with only transport
+            // controls.
+            mMediaSession.setActive(false);
+            mMediaSession.setFlags(MediaSessionCompat.FLAG_HANDLES_TRANSPORT_CONTROLS);
+            mMediaSession.setActive(true);
 
-        MediaWrapper media = getCurrentMedia();
-        if (mRemoteControlClient != null && media != null) {
-            MetadataEditor editor = mRemoteControlClient.editMetadata(true);
-            if (media.getNowPlaying() != null) {
-                editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUM, "");
-                editor.putString(MediaMetadataRetriever.METADATA_KEY_ARTIST, "");
-                editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUMARTIST, media.getNowPlaying());
-            } else {
-                editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUMARTIST, "");
-                editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUM, Util.getMediaAlbum(this, media));
-                editor.putString(MediaMetadataRetriever.METADATA_KEY_ARTIST, Util.getMediaArtist(this, media));
-            }
-            editor.putString(MediaMetadataRetriever.METADATA_KEY_GENRE, Util.getMediaGenre(this, media));
-            editor.putString(MediaMetadataRetriever.METADATA_KEY_TITLE, media.getTitle());
-            editor.putLong(MediaMetadataRetriever.METADATA_KEY_DURATION, media.getLength());
+        }
+    }
 
-            // Copy the cover bitmap because the RemonteControlClient can recycle its artwork bitmap.
-            Bitmap cover = AudioUtil.getCover(this, media, 512);
-            if (cover != null && cover.getConfig() != null) //In case of format not supported
-                editor.putBitmap(MetadataEditor.BITMAP_KEY_ARTWORK, (cover.copy(cover.getConfig(), false)));
-
-            editor.apply();
+    private final class MediaSessionCallback extends MediaSessionCompat.Callback {
+        @Override
+        public void onPlay() {
+            play();
+        }
+        @Override
+        public void onPause() {
+            pause();
         }
 
+        @Override
+        public void onStop() {
+            stop();
+        }
+
+        @Override
+        public void onSkipToNext() {
+            next();
+        }
+
+        @Override
+        public void onSkipToPrevious() {
+            previous();
+        }
+    }
+
+    protected void updateMetadata() {
+        MediaWrapper media = getCurrentMedia();
+        if (media == null)
+            return;
+        if (mMediaSession == null)
+            initMediaSession();
+        String title = media.getNowPlaying();
+        if (title == null)
+            title = media.getTitle();
+        Bitmap cover = AudioUtil.getCover(this, media, 512);
+        MediaMetadataCompat.Builder bob = new MediaMetadataCompat.Builder();
+        bob.putString(MediaMetadataCompat.METADATA_KEY_TITLE, title)
+            .putString(MediaMetadataCompat.METADATA_KEY_GENRE, MediaUtils.getMediaGenre(this, media))
+                .putLong(MediaMetadataCompat.METADATA_KEY_TRACK_NUMBER, media.getTrackNumber())
+                .putString(MediaMetadataCompat.METADATA_KEY_ALBUM_ARTIST, MediaUtils.getMediaReferenceArtist(this, media))
+                .putString(MediaMetadataCompat.METADATA_KEY_ALBUM, MediaUtils.getMediaAlbum(this, media))
+                .putLong(MediaMetadataCompat.METADATA_KEY_DURATION, media.getLength());
+        if (cover != null && cover.getConfig() != null) //In case of format not supported
+                bob.putBitmap(MediaMetadataCompat.METADATA_KEY_ALBUM_ART, cover.copy(cover.getConfig(), false));
+        mMediaSession.setMetadata(bob.build());
+
         //Send metadata to Pebble watch
-        if (media != null && mPebbleEnabled) {
+        if (mPebbleEnabled) {
             final Intent i = new Intent("com.getpebble.action.NOW_PLAYING");
-            i.putExtra("artist", Util.getMediaArtist(this, media));
-            i.putExtra("album", Util.getMediaAlbum(this, media));
+            i.putExtra("artist", MediaUtils.getMediaArtist(this, media));
+            i.putExtra("album", MediaUtils.getMediaAlbum(this, media));
             i.putExtra("track", media.getTitle());
             sendBroadcast(i);
         }
     }
 
+    protected void publishState(int state) {
+        if (mMediaSession == null)
+            return;
+        PlaybackStateCompat.Builder bob = new PlaybackStateCompat.Builder();
+        bob.setActions(PLAYBACK_ACTIONS);
+        switch (state) {
+            case MediaPlayer.Event.Playing:
+                bob.setState(PlaybackStateCompat.STATE_PLAYING, -1, 1);
+                break;
+            case MediaPlayer.Event.Stopped:
+                bob.setState(PlaybackStateCompat.STATE_STOPPED, -1, 0);
+                break;
+            default:
+            bob.setState(PlaybackStateCompat.STATE_PAUSED, -1, 0);
+        }
+        PlaybackStateCompat pbState = bob.build();
+        mMediaSession.setPlaybackState(pbState);
+        mMediaSession.setActive(state != PlaybackStateCompat.STATE_STOPPED);
+    }
+
     private void notifyTrackChanged() {
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
-        showNotification();
+        updateMetadata();
         updateWidget();
         broadcastMetadata();
-        updateRemoteControlClientMetadata();
     }
 
     private void onMediaChanged() {
@@ -1163,7 +1148,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     @MainThread
     public void previous() {
         int size = mMediaList.size();
-        if (size > 1) {
+        if (hasPrevious() && mCurrentIndex > 0 && mMediaPlayer.getTime() < 2000l) {
             mCurrentIndex = mPrevIndex;
             if (mPrevious.size() > 0)
                 mPrevious.pop();
@@ -1201,14 +1186,14 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     private void updateWidgetState() {
-        Intent i = new Intent(ACTION_WIDGET_UPDATE);
+        Intent i = new Intent(VLCAppWidgetProvider.ACTION_WIDGET_UPDATE);
 
         if (hasCurrentMedia()) {
             final MediaWrapper media = getCurrentMedia();
             i.putExtra("title", media.getTitle());
             i.putExtra("artist", media.isArtistUnknown() && media.getNowPlaying() != null ?
                     media.getNowPlaying()
-                    : Util.getMediaArtist(this, media));
+                    : MediaUtils.getMediaArtist(this, media));
         }
         else {
             i.putExtra("title", getString(R.string.widget_default_text));
@@ -1220,7 +1205,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     private void updateWidgetCover() {
-        Intent i = new Intent(ACTION_WIDGET_UPDATE_COVER);
+        Intent i = new Intent(VLCAppWidgetProvider.ACTION_WIDGET_UPDATE_COVER);
 
         Bitmap cover = hasCurrentMedia() ? AudioUtil.getCover(this, getCurrentMedia(), 64) : null;
         i.putExtra("cover", cover);
@@ -1238,7 +1223,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         updateWidgetState();
 
         mWidgetPositionTimestamp = timestamp;
-        Intent i = new Intent(ACTION_WIDGET_UPDATE_POSITION);
+        Intent i = new Intent(VLCAppWidgetProvider.ACTION_WIDGET_UPDATE_POSITION);
         i.putExtra("position", pos);
         sendBroadcast(i);
     }
@@ -1383,7 +1368,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     @MainThread
     public String getAlbum() {
         if (hasCurrentMedia())
-            return Util.getMediaAlbum(PlaybackService.this, getCurrentMedia());
+            return MediaUtils.getMediaAlbum(PlaybackService.this, getCurrentMedia());
         else
             return null;
     }
@@ -1394,7 +1379,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             final MediaWrapper media = getCurrentMedia();
             return media.isArtistUnknown() && media.getNowPlaying() != null ?
                     media.getNowPlaying()
-                    : Util.getMediaArtist(PlaybackService.this, media);
+                    : MediaUtils.getMediaArtist(PlaybackService.this, media);
         } else
             return null;
     }
@@ -1402,7 +1387,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     @MainThread
     public String getArtistPrev() {
         if (mPrevIndex != -1)
-            return Util.getMediaArtist(PlaybackService.this, mMediaList.getMedia(mPrevIndex));
+            return MediaUtils.getMediaArtist(PlaybackService.this, mMediaList.getMedia(mPrevIndex));
         else
             return null;
     }
@@ -1410,7 +1395,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     @MainThread
     public String getArtistNext() {
         if (mNextIndex != -1)
-            return Util.getMediaArtist(PlaybackService.this, mMediaList.getMedia(mNextIndex));
+            return MediaUtils.getMediaArtist(PlaybackService.this, mMediaList.getMedia(mNextIndex));
         else
             return null;
     }
@@ -1516,11 +1501,18 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         load(mediaList, position);
     }
 
+    @MainThread
+    public void loadUri(Uri uri) {
+        String path = uri.getPath();
+        if (TextUtils.equals(uri.getScheme(), "content")) {
+            path = "file://"+ FileUtils.getPathFromURI(uri);
+        }
+        loadLocation(path);
+    }
+
     @MainThread
     public void loadLocation(String mediaPath) {
-        ArrayList <String> arrayList = new ArrayList<String>();
-        arrayList.add(mediaPath);
-        loadLocations(arrayList, 0);
+        loadLocations(Collections.singletonList(mediaPath), 0);
     }
 
     @MainThread
@@ -1591,6 +1583,9 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if (mw == null)
             return;
 
+        /* Pausable and seekable are true by default */
+        mParsed = false;
+        mPausable = mSeekable = true;
         final Media media = new Media(VLCInstance.get(), mw.getUri());
         VLCOptions.setMediaOptions(media, this, flags | mw.getFlags());
         media.setEventListener(mMediaListener);
@@ -1633,6 +1628,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         mCurrentIndex = index;
 
         notifyTrackChanged();
+        showNotification();
     }
 
     /**
@@ -1667,12 +1663,14 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public void moveItem(int positionStart, int positionEnd) {
         mMediaList.move(positionStart, positionEnd);
         PlaybackService.this.saveMediaList();
+        Log.d(TAG, "moveItem "+positionStart+" -> "+positionEnd);
     }
 
     @MainThread
     public void remove(int position) {
         mMediaList.remove(position);
         onMediaListChanged();
+        Log.d(TAG, "remove "+position);
     }
 
     @MainThread
@@ -1979,4 +1977,21 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             startService(context);
         }
     }
+
+    int mPhoneEvents = PhoneStateListener.LISTEN_CALL_STATE;
+    PhoneStateListener mPhoneStateListener;
+
+    private void initPhoneListener() {
+        mPhoneStateListener = new PhoneStateListener(){
+            @Override
+            public void onCallStateChanged(int state, String incomingNumber) {
+                if (!mMediaPlayer.isPlaying() || !hasCurrentMedia())
+                    return;
+                if (state == TelephonyManager.CALL_STATE_RINGING || state == TelephonyManager.CALL_STATE_OFFHOOK)
+                    pause();
+                else if (state == TelephonyManager.CALL_STATE_IDLE)
+                    play();
+            }
+        };
+    }
 }
