@@ -32,32 +32,44 @@ import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
 import android.preference.PreferenceManager;
+import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentManager;
 import android.support.v7.app.ActionBar;
 import android.support.v7.app.AppCompatActivity;
-import android.support.v4.app.Fragment;
 import android.support.v7.widget.Toolbar;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
 import android.widget.TextView;
 
+import com.android.widget.SlidingPaneLayout;
+
 import org.videolan.vlc.BuildConfig;
-import org.videolan.vlc.MediaLibrary;
+import org.videolan.vlc.gui.helpers.UiTools;
+import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.gui.audio.AudioPlayer;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.interfaces.IRefreshable;
+import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.WeakHandler;
-import org.videolan.vlc.widget.HackyDrawerLayout;
-import com.android.widget.SlidingPaneLayout;
+import org.videolan.vlc.gui.view.HackyDrawerLayout;
 
 public class AudioPlayerContainerActivity extends AppCompatActivity implements PlaybackService.Client.Callback  {
 
     public static final String TAG = "VLC/AudioPlayerContainerActivity";
-    public static final String ACTION_SHOW_PLAYER = "org.videolan.vlc.gui.ShowPlayer";
+    public static final String ACTION_SHOW_PLAYER = Strings.buildPkgString("gui.ShowPlayer");
+
+    protected static final String ID_VIDEO = "video";
+    protected static final String ID_AUDIO = "audio";
+    protected static final String ID_NETWORK = "network";
+    protected static final String ID_DIRECTORIES = "directories";
+    protected static final String ID_HISTORY = "history";
+    protected static final String ID_MRL = "mrl";
+    protected static final String ID_PREFERENCES = "preferences";
+    protected static final String ID_ABOUT = "about";
 
     protected ActionBar mActionBar;
     protected Toolbar mToolbar;
@@ -155,11 +167,11 @@ public class AudioPlayerContainerActivity extends AppCompatActivity implements P
             ((IRefreshable) current).refresh();
         else
             MediaLibrary.getInstance().scanMediaItems();
-        Fragment fragment = fm.findFragmentByTag(SidebarAdapter.SidebarEntry.ID_AUDIO);
+        Fragment fragment = fm.findFragmentByTag(ID_AUDIO);
         if (fragment != null && !fragment.equals(current)) {
             ((MediaBrowserFragment)fragment).clear();
         }
-        fragment = fm.findFragmentByTag(SidebarAdapter.SidebarEntry.ID_VIDEO);
+        fragment = fm.findFragmentByTag(ID_VIDEO);
         if (fragment != null && !fragment.equals(current)) {
             ((MediaBrowserFragment)fragment).clear();
         }
@@ -282,7 +294,7 @@ public class AudioPlayerContainerActivity extends AppCompatActivity implements P
 
         @Override
         public void onPanelOpened() {
-            int resId = Util.getResourceFromAttribute(AudioPlayerContainerActivity.this, R.attr.shadow_bottom_9patch);
+            int resId = UiTools.getResourceFromAttribute(AudioPlayerContainerActivity.this, R.attr.shadow_bottom_9patch);
             if (resId != 0)
                 mSlidingPane.setShadowResource(resId);
             mAudioPlayer.setHeaderVisibilities(false, false, true, true, true, false);
