@@ -26,12 +26,14 @@ import android.content.Context;
 import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.SharedPreferences.Editor;
+import android.content.pm.PackageManager;
 import android.database.Cursor;
 import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
+import android.support.design.widget.NavigationView;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentActivity;
 import android.support.v4.app.FragmentManager;
@@ -40,6 +42,7 @@ import android.support.v4.view.GravityCompat;
 import android.support.v4.view.MenuItemCompat;
 import android.support.v7.app.ActionBarDrawerToggle;
 import android.support.v7.widget.SearchView;
+import android.text.TextUtils;
 import android.util.Log;
 import android.view.KeyEvent;
 import android.view.Menu;
@@ -49,59 +52,61 @@ import android.view.View;
 import android.view.Window;
 import android.view.WindowManager;
 import android.view.inputmethod.InputMethodManager;
-import android.widget.AdapterView;
-import android.widget.AdapterView.OnItemClickListener;
 import android.widget.FilterQueryProvider;
-import android.widget.ListView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
 
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.BuildConfig;
-import org.videolan.vlc.MediaDatabase;
-import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
 import org.videolan.vlc.gui.audio.AudioBrowserFragment;
 import org.videolan.vlc.gui.browser.BaseBrowserFragment;
+import org.videolan.vlc.gui.browser.FileBrowserFragment;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.browser.NetworkBrowserFragment;
+import org.videolan.vlc.gui.helpers.SearchSuggestionsAdapter;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.network.MRLPanelFragment;
 import org.videolan.vlc.gui.preferences.PreferencesActivity;
 import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.gui.video.VideoListAdapter;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
+import org.videolan.vlc.gui.view.HackyDrawerLayout;
 import org.videolan.vlc.interfaces.IRefreshable;
 import org.videolan.vlc.interfaces.ISortable;
+import org.videolan.vlc.media.MediaDatabase;
+import org.videolan.vlc.media.MediaLibrary;
+import org.videolan.vlc.media.MediaUtils;
+import org.videolan.vlc.util.Permissions;
 import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.WeakHandler;
-import org.videolan.vlc.widget.HackyDrawerLayout;
 
-public class MainActivity extends AudioPlayerContainerActivity implements OnItemClickListener, SearchSuggestionsAdapter.SuggestionDisplay, FilterQueryProvider {
+public class MainActivity extends AudioPlayerContainerActivity implements SearchSuggestionsAdapter.SuggestionDisplay, FilterQueryProvider, NavigationView.OnNavigationItemSelectedListener {
     public final static String TAG = "VLC/MainActivity";
 
     private static final String PREF_FIRST_RUN = "first_run";
 
     private static final int ACTIVITY_RESULT_PREFERENCES = 1;
+    private static final int ACTIVITY_RESULT_OPEN = 2;
     private static final int ACTIVITY_SHOW_INFOLAYOUT = 2;
     private static final int ACTIVITY_SHOW_PROGRESSBAR = 3;
     private static final int ACTIVITY_HIDE_PROGRESSBAR = 4;
     private static final int ACTIVITY_SHOW_TEXTINFO = 5;
 
+
     MediaLibrary mMediaLibrary;
 
-    private SidebarAdapter mSidebarAdapter;
     private HackyDrawerLayout mDrawerLayout;
-    private ListView mListView;
+    private NavigationView mNavigationView;
     private ActionBarDrawerToggle mDrawerToggle;
 
     private View mInfoLayout;
     private ProgressBar mInfoProgress;
     private TextView mInfoText;
-    private String mCurrentFragment;
+    private int mCurrentFragment;
 
 
     private int mVersionNumber = -1;
@@ -136,6 +141,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             Util.commitPreferences(editor);
         }
 
+        Permissions.checkReadStoragePermission(this, false);
+
         mMediaLibrary = MediaLibrary.getInstance();
         if (mMediaLibrary.getMediaItems().isEmpty()) {
             if (mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true))
@@ -149,17 +156,14 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         setContentView(R.layout.main);
 
         mDrawerLayout = (HackyDrawerLayout) findViewById(R.id.root_container);
-        mListView = (ListView)findViewById(R.id.sidelist);
-        mListView.setFooterDividersEnabled(true);
-        mSidebarAdapter = new SidebarAdapter(this);
-        mListView.setAdapter(mSidebarAdapter);
+        setupNavigationView();
 
         initAudioPlayerContainerActivity();
 
         if (savedInstanceState != null){
-            mCurrentFragment = savedInstanceState.getString("current");
-            if (mCurrentFragment != null)
-                mSidebarAdapter.setCurrentFragment(mCurrentFragment);
+            mCurrentFragment = savedInstanceState.getInt("current");
+            if (mCurrentFragment > 0)
+                mNavigationView.setCheckedItem(mCurrentFragment);
         }
 
 
@@ -187,8 +191,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         // set a custom shadow that overlays the main content when the drawer opens
         mDrawerLayout.setDrawerShadow(R.drawable.drawer_shadow, GravityCompat.START);
 
-        mListView.setOnItemClickListener(this);
-
         if (mFirstRun) {
             /*
              * The sliding menu is automatically opened when the user closes
@@ -198,7 +200,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             mHandler.postDelayed(new Runnable() {
                 @Override
                 public void run() {
-                    mDrawerLayout.openDrawer(mListView);
+                    mDrawerLayout.openDrawer(mNavigationView);
                 }
             }, 500);
         }
@@ -207,6 +209,36 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         reloadPreferences();
     }
 
+    private void setupNavigationView() {
+        mNavigationView = (NavigationView) findViewById(R.id.navigation);
+        mNavigationView.setNavigationItemSelectedListener(this);
+        if (TextUtils.equals(BuildConfig.FLAVOR_target, "chrome")) {
+            MenuItem item = mNavigationView.getMenu().findItem(R.id.nav_directories);
+            item.setTitle(R.string.open);
+        }
+
+        if (AndroidUtil.isLolliPopOrLater())
+            mNavigationView.setPadding(0, mNavigationView.getPaddingTop()/2, 0, 0);
+    }
+
+    @Override
+    public void onRequestPermissionsResult(int requestCode,
+                                           String permissions[], int[] grantResults) {
+        switch (requestCode) {
+            case Permissions.PERMISSION_STORAGE_TAG:
+                // If request is cancelled, the result arrays are empty.
+                if (grantResults.length > 0
+                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
+                    forceRefresh(getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder));
+                } else {
+                    Permissions.showStoragePermissionDialog(this, false);
+                }
+                return;
+            // other 'case' lines to check for other
+            // permissions this app might request
+        }
+    }
+
     @Override
     protected void onPostCreate(Bundle savedInstanceState) {
         super.onPostCreate(savedInstanceState);
@@ -229,6 +261,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             mMediaLibrary.scanMediaItems();
         if (mSlidingPane.getState() == mSlidingPane.STATE_CLOSED)
             mActionBar.hide();
+        mNavigationView.setCheckedItem(mCurrentFragment);
+        mCurrentFragment = mSettings.getInt("fragment_id", R.id.nav_video);
     }
 
     @Override
@@ -238,7 +272,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         // Figure out if currently-loaded fragment is a top-level fragment.
         Fragment current = getSupportFragmentManager()
                 .findFragmentById(R.id.fragment_placeholder);
-        boolean found = (current == null) || SidebarAdapter.sidebarFragments.contains(current.getTag());
 
         /**
          * Restore the last view.
@@ -252,17 +285,11 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
          * It will try to remove() the currently displayed fragment
          * (i.e. tracks) and replace it with a blank screen. (stuck menu bug)
          */
-        if (current == null || (!current.getTag().equals(mCurrentFragment) && found)) {
-            Log.d(TAG, "Reloading displayed fragment");
-            if (mCurrentFragment == null)
-                mCurrentFragment = "video";
-            if (!SidebarAdapter.sidebarFragments.contains(mCurrentFragment)) {
-                Log.d(TAG, "Unknown fragment \"" + mCurrentFragment + "\", resetting to video");
-                mCurrentFragment = "video";
-            }
+        if (current == null) {
+            mNavigationView.setCheckedItem(mCurrentFragment);
             Fragment ff = getFragment(mCurrentFragment);
             FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-            ft.replace(R.id.fragment_placeholder, ff, mCurrentFragment);
+            ft.replace(R.id.fragment_placeholder, ff, getTag(mCurrentFragment));
             ft.commit();
         }
     }
@@ -279,7 +306,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         mMediaLibrary.stop();
         /* Save the tab status in pref */
         SharedPreferences.Editor editor = mSettings.edit();
-        editor.putString("fragment", mCurrentFragment);
+        editor.putInt("fragment_id", mCurrentFragment);
         Util.commitPreferences(editor);
 
         mFocusedPrior = 0;
@@ -287,7 +314,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
 
     protected void onSaveInstanceState(Bundle outState) {
         super.onSaveInstanceState(outState);
-        outState.putString("current", mCurrentFragment);
+        outState.putInt("current", mCurrentFragment);
     }
 
     @Override
@@ -300,10 +327,10 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
     @Override
     public void onBackPressed() {
         /* Close the menu first */
-        if(mDrawerLayout.isDrawerOpen(mListView)) {
+        if(mDrawerLayout.isDrawerOpen(mNavigationView)) {
             if (mFocusedPrior != 0)
                 requestFocusOnSearch();
-            mDrawerLayout.closeDrawer(mListView);
+            mDrawerLayout.closeDrawer(mNavigationView);
             return;
         }
 
@@ -311,26 +338,37 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         if (slideDownAudioPlayer())
             return;
 
-        if (mCurrentFragment!= null) {
-            // If it's the directory view, a "backpressed" action shows a parent.
-            if (mCurrentFragment.equals(SidebarEntry.ID_NETWORK) || mCurrentFragment.equals(SidebarEntry.ID_DIRECTORIES)){
-                BaseBrowserFragment browserFragment = (BaseBrowserFragment) getSupportFragmentManager()
-                        .findFragmentById(R.id.fragment_placeholder);
-                if (browserFragment != null) {
-                    browserFragment.goBack();
-                    return;
-                }
+        // If it's the directory view, a "backpressed" action shows a parent.
+        if (mCurrentFragment == R.id.nav_network || mCurrentFragment == R.id.nav_directories){
+            BaseBrowserFragment browserFragment = (BaseBrowserFragment) getSupportFragmentManager()
+                    .findFragmentById(R.id.fragment_placeholder);
+            if (browserFragment != null) {
+                browserFragment.goBack();
+                return;
             }
         }
         finish();
     }
 
-    private Fragment getFragment(String id)
+    private Fragment getFragment(int id)
     {
-        Fragment frag = getSupportFragmentManager().findFragmentByTag(id);
+        Fragment frag = getSupportFragmentManager().findFragmentByTag(getTag(id));
         if (frag != null)
             return frag;
-        return mSidebarAdapter.fetchFragment(id);
+        switch (id) {
+            case R.id.nav_audio:
+                return new AudioBrowserFragment();
+            case R.id.nav_directories:
+                return new FileBrowserFragment();
+            case R.id.nav_history:
+                return new HistoryFragment();
+            case R.id.nav_mrl:
+                return new MRLPanelFragment();
+            case R.id.nav_network:
+                return new NetworkBrowserFragment();
+            default:
+                return new VideoGridFragment();
+        }
     }
 
     private static void ShowFragment(FragmentActivity activity, String tag, Fragment fragment, String previous) {
@@ -488,12 +526,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                 break;
             // Refresh
             case R.id.ml_menu_refresh:
-                if (!mMediaLibrary.isWorking()) {
-                    if(current != null && current instanceof IRefreshable)
-                        ((IRefreshable) current).refresh();
-                    else
-                        mMediaLibrary.scanMediaItems(true);
-                }
+                forceRefresh(current);
                 break;
             // Restore last playlist
             case R.id.ml_menu_last_playlist:
@@ -516,8 +549,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                 }
                 break;
             case R.id.ml_menu_clean:
-                if (getFragment(mCurrentFragment) instanceof MRLPanelFragment)
-                    ((MRLPanelFragment)getFragment(mCurrentFragment)).clearHistory();
+                if (current instanceof MRLPanelFragment)
+                    ((MRLPanelFragment)current).clearHistory();
                 break;
             case R.id.ml_menu_save:
                 if (current == null)
@@ -526,10 +559,19 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                 item.setIcon(R.drawable.ic_menu_bookmark_w);
                 break;
         }
-        mDrawerLayout.closeDrawer(mListView);
+        mDrawerLayout.closeDrawer(mNavigationView);
         return super.onOptionsItemSelected(item);
     }
 
+    private void forceRefresh(Fragment current) {
+        if (!mMediaLibrary.isWorking()) {
+            if(current != null && current instanceof IRefreshable)
+                ((IRefreshable) current).refresh();
+            else
+                mMediaLibrary.scanMediaItems(true);
+        }
+    }
+
     @Override
     protected void onActivityResult(int requestCode, int resultCode, Intent data) {
         super.onActivityResult(requestCode, resultCode, data);
@@ -543,6 +585,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                     startActivity(intent);
                 }
             }
+        } else if (requestCode == ACTIVITY_RESULT_OPEN && resultCode == RESULT_OK){
+            MediaUtils.openUri(this, data.getData());
         }
     }
 
@@ -571,7 +615,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                     } else if (pane == mSlidingPane.STATE_OPENED) {
                         v.setNextFocusDownId(R.id.header_play_pause);
                     } else if (pane ==
-                        mSlidingPane.STATE_OPENED_ENTIRELY) {
+                            mSlidingPane.STATE_OPENED_ENTIRELY) {
                         v.setNextFocusDownId(r);
                     }
                 }
@@ -600,7 +644,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                 }
             }
         } else {
-           playPause.setNextFocusUpId(R.id.ml_menu_search);
+            playPause.setNextFocusUpId(R.id.ml_menu_search);
         }
     }
 
@@ -635,12 +679,12 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         if (v == null)
             return super.onKeyUp(keyCode, event);
         if ((mActionBarIconId == -1) &&
-            (v.getId() == -1)  &&
-            (v.getNextFocusDownId() == -1) &&
-            (v.getNextFocusUpId() == -1) &&
-            (v.getNextFocusLeftId() == -1) &&
-            (v.getNextFocusRightId() == -1)) {
-            mActionBarIconId = Util.generateViewId();
+                (v.getId() == -1)  &&
+                (v.getNextFocusDownId() == -1) &&
+                (v.getNextFocusUpId() == -1) &&
+                (v.getNextFocusLeftId() == -1) &&
+                (v.getNextFocusRightId() == -1)) {
+            mActionBarIconId = UiTools.generateViewId();
             v.setId(mActionBarIconId);
             v.setNextFocusUpId(mActionBarIconId);
             v.setNextFocusDownId(mActionBarIconId);
@@ -655,7 +699,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
     }
 
     private void reloadPreferences() {
-        mCurrentFragment = mSettings.getString("fragment", "video");
+        mCurrentFragment = mSettings.getInt("fragment_id", R.id.nav_video);
     }
 
     @Override
@@ -745,45 +789,80 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         removeTipViewIfDisplayed();
     }
 
+
     @Override
-    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
-        SidebarAdapter.SidebarEntry entry = (SidebarEntry) mListView.getItemAtPosition(position);
+    public boolean onNavigationItemSelected(MenuItem item) {
+        int id = item.getItemId();
         Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
 
-        if(current == null || (entry != null && current.getTag().equals(entry.id))) { /* Already selected */
+        if(current == null || (item != null && mCurrentFragment == id)) { /* Already selected */
             if (mFocusedPrior != 0)
                 requestFocusOnSearch();
-            mDrawerLayout.closeDrawer(mListView);
-            return;
+            mDrawerLayout.closeDrawer(mNavigationView);
+            return false;
         }
 
         // This should not happen
-        if(entry == null || entry.id == null)
-            return;
-
-        if (entry.type == SidebarEntry.TYPE_FRAGMENT) {
+        if(item == null)
+            return false;
 
+        String tag = getTag(id);
+        switch (id){
+            case R.id.nav_about:
+                showSecondaryFragment(SecondaryActivity.ABOUT);
+                break;
+            case R.id.nav_settings:
+                startActivityForResult(new Intent(this, PreferencesActivity.class), ACTIVITY_RESULT_PREFERENCES);
+                break;
+            case R.id.nav_directories:
+                if (TextUtils.equals(BuildConfig.FLAVOR_target, "chrome")) {
+                    Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
+                    intent.setType("audio/* video/*");
+                    startActivityForResult(intent, ACTIVITY_RESULT_OPEN);
+                    mDrawerLayout.closeDrawer(mNavigationView);
+                    return true;
+                }
+            default:
                 /* Slide down the audio player */
-            slideDownAudioPlayer();
+                slideDownAudioPlayer();
 
                 /* Switch the fragment */
-            Fragment fragment = getFragment(entry.id);
-            if (fragment instanceof MediaBrowserFragment)
-                ((MediaBrowserFragment)fragment).setReadyToDisplay(false);
-            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-            ft.replace(R.id.fragment_placeholder, fragment, entry.id);
-            ft.addToBackStack(mCurrentFragment);
-            ft.commit();
-            mCurrentFragment = entry.id;
-            mSidebarAdapter.setCurrentFragment(mCurrentFragment);
+                Fragment fragment = getFragment(id);
+                if (fragment instanceof MediaBrowserFragment)
+                    ((MediaBrowserFragment)fragment).setReadyToDisplay(false);
+                FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
+                ft.replace(R.id.fragment_placeholder, fragment, tag);
+                ft.addToBackStack(getTag(mCurrentFragment));
+                ft.commit();
+                mCurrentFragment = id;
 
-            if (mFocusedPrior != 0)
-                requestFocusOnSearch();
-        } else if (entry.type == SidebarEntry.TYPE_SECONDARY_FRAGMENT)
-            showSecondaryFragment(SecondaryActivity.ABOUT);
-        else if (entry.attributeID == R.attr.ic_menu_preferences)
-            startActivityForResult(new Intent(this, PreferencesActivity.class), ACTIVITY_RESULT_PREFERENCES);
-        mDrawerLayout.closeDrawer(mListView);
+                if (mFocusedPrior != 0)
+                    requestFocusOnSearch();
+        }
+        mDrawerLayout.closeDrawer(mNavigationView);
+        mNavigationView.setCheckedItem(mCurrentFragment);
+        return true;
+    }
+
+    private String getTag(int id){
+        switch (id){
+            case R.id.nav_about:
+                return ID_ABOUT;
+            case R.id.nav_settings:
+                return ID_PREFERENCES;
+            case R.id.nav_audio:
+                return ID_AUDIO;
+            case R.id.nav_directories:
+                return ID_DIRECTORIES;
+            case R.id.nav_history:
+                return ID_HISTORY;
+            case R.id.nav_mrl:
+                return ID_MRL;
+            case R.id.nav_network:
+                return ID_NETWORK;
+            default:
+                return ID_VIDEO;
+        }
     }
 
     private void requestFocusOnSearch() {
