@@ -26,8 +26,8 @@ import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
-import android.support.design.widget.Snackbar;
 import android.support.design.widget.FloatingActionButton;
+import android.support.design.widget.Snackbar;
 import android.support.design.widget.TabLayout;
 import android.support.v4.view.ViewPager;
 import android.view.ContextMenu;
@@ -52,22 +52,24 @@ import android.widget.TextView;
 import org.videolan.libvlc.Media;
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.MediaBrowser;
-import org.videolan.vlc.MediaDatabase;
-import org.videolan.vlc.MediaLibrary;
-import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
+import org.videolan.vlc.gui.helpers.AudioUtil;
+import org.videolan.vlc.gui.helpers.MediaComparators;
+import org.videolan.vlc.gui.view.SwipeRefreshLayout;
 import org.videolan.vlc.interfaces.IBrowser;
+import org.videolan.vlc.media.MediaDatabase;
+import org.videolan.vlc.media.MediaLibrary;
+import org.videolan.vlc.media.MediaUtils;
+import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
-import org.videolan.vlc.util.Util;
+import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.WeakHandler;
-import org.videolan.vlc.widget.SwipeRefreshLayout;
 
-import java.io.File;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -324,7 +326,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
             i.putExtra("fragment", "albumsSongs");
             i.putParcelableArrayListExtra("list", mediaList);
-            i.putExtra("filter", Util.getMediaArtist(activity, mediaList.get(0)));
+            i.putExtra("filter", MediaUtils.getMediaArtist(activity, mediaList.get(0)));
             startActivity(i);
         }
     };
@@ -336,7 +338,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
             i.putExtra("fragment", SecondaryActivity.ALBUM);
             i.putParcelableArrayListExtra("list", mediaList);
-            i.putExtra("filter", Util.getMediaAlbum(getActivity(), mediaList.get(0)));
+            i.putExtra("filter", MediaUtils.getMediaAlbum(getActivity(), mediaList.get(0)));
             startActivity(i);
         }
     };
@@ -348,7 +350,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
             i.putExtra("fragment", SecondaryActivity.ALBUMS_SONGS);
             i.putParcelableArrayListExtra("list", mediaList);
-            i.putExtra("filter", Util.getMediaGenre(getActivity(), mediaList.get(0)));
+            i.putExtra("filter", MediaUtils.getMediaGenre(getActivity(), mediaList.get(0)));
             startActivity(i);
         }
     };
@@ -375,10 +377,12 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
         MenuInflater inflater = getActivity().getMenuInflater();
         inflater.inflate(R.menu.audio_list_browser, menu);
-        setContextMenuItems(menu, v);
+
+        int position = ((AdapterContextMenuInfo) menuInfo).position;
+        setContextMenuItems(menu, position);
     }
 
-    private void setContextMenuItems(Menu menu, View v) {
+    private void setContextMenuItems(Menu menu, int position) {
         final int pos = mViewPager.getCurrentItem();
         if (pos != MODE_SONG) {
             menu.setGroupVisible(R.id.songs_view_only, false);
@@ -388,6 +392,17 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             menu.findItem(R.id.audio_list_browser_play).setVisible(true);
         if (pos != MODE_SONG && pos != MODE_PLAYLIST)
             menu.findItem(R.id.audio_list_browser_delete).setVisible(false);
+        else {
+            MenuItem item = menu.findItem(R.id.audio_list_browser_delete);
+            AudioBrowserListAdapter adapter = pos == MODE_SONG ? mSongsAdapter : mPlaylistAdapter;
+            AudioBrowserListAdapter.ListItem mediaItem = adapter.getItem(position);
+            if (pos == MODE_PLAYLIST && MediaDatabase.getInstance().playlistExists(mediaItem.mTitle))
+                item.setVisible(true);
+            else {
+                String location = mediaItem.mMediaList.get(0).getLocation();
+                item.setVisible(FileUtils.canWrite(location));
+            }
+        }
         if (!AndroidDevices.isPhone())
             menu.setGroupVisible(R.id.phone_only, false);
     }
@@ -679,11 +694,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                     MediaWrapper mw = fragment.mSongsAdapter.getItem(msg.arg1).mMediaList.get(0);
                     final String path = mw.getUri().getPath();
                     //Let's keep this toast while duration is not set correctly
-                    VLCApplication.runBackground(new Runnable() {
-                        public void run() {
-                            Util.recursiveDelete(VLCApplication.getAppContext(), new File(path));
-                        }
-                    });
+                    FileUtils.asyncRecursiveDelete(path);
                     fragment.mMediaLibrary.getMediaItems().remove(mw);
                     refresh(fragment, path);
                     break;
@@ -820,7 +831,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
 
                 PopupMenu popupMenu = new PopupMenu(getActivity(), anchor);
                 popupMenu.getMenuInflater().inflate(R.menu.audio_list_browser, popupMenu.getMenu());
-                setContextMenuItems(popupMenu.getMenu(), anchor);
+                setContextMenuItems(popupMenu.getMenu(), position);
 
                 popupMenu.setOnMenuItemClickListener(new OnMenuItemClickListener() {
                     @Override
