@@ -36,18 +36,19 @@ import android.view.View;
 import android.view.ViewGroup;
 import android.widget.BaseAdapter;
 import android.widget.ImageView;
-import android.widget.LinearLayout;
 import android.widget.SectionIndexer;
 import android.widget.TextView;
 
 import org.videolan.vlc.BR;
-import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.gui.AsyncImageLoader;
+import org.videolan.vlc.gui.helpers.AsyncImageLoader;
+import org.videolan.vlc.gui.helpers.AudioUtil;
+import org.videolan.vlc.gui.helpers.MediaComparators;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.interfaces.IAudioClickHandler;
-import org.videolan.vlc.util.BitmapUtil;
-import org.videolan.vlc.util.Util;
+import org.videolan.vlc.media.MediaUtils;
+import org.videolan.vlc.media.MediaWrapper;
 
 import java.util.ArrayList;
 import java.util.Collections;
@@ -139,12 +140,12 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
     public void add(String title, String subTitle, MediaWrapper media, String key) {
         if(title == null) return;
         title = title.trim();
+        if(subTitle != null) subTitle = subTitle.trim();
         String mediaKey;
         if (key == null)
-            mediaKey = title.toLowerCase(Locale.getDefault());
+            mediaKey = (title + subTitle).toLowerCase(Locale.getDefault());
         else
             mediaKey = key.trim().toLowerCase(Locale.getDefault());
-        if(subTitle != null) subTitle = subTitle.trim();
         if (mMediaItemMap.containsKey(mediaKey))
             mMediaItemMap.get(mediaKey).mMediaList.add(media);
         else {
@@ -164,17 +165,17 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
                 for (MediaWrapper media : list) {
                     switch (type){
                         case TYPE_ALBUMS:
-                            title = Util.getMediaAlbum(mContext, media);
-                            subTitle = Util.getMediaReferenceArtist(mContext, media);
+                            title = MediaUtils.getMediaAlbum(mContext, media);
+                            subTitle = MediaUtils.getMediaReferenceArtist(mContext, media);
                             key = null;
                             break;
                         case TYPE_ARTISTS:
-                            title = Util.getMediaReferenceArtist(mContext, media);
+                            title = MediaUtils.getMediaReferenceArtist(mContext, media);
                             subTitle = null;
                             key = null;
                             break;
                         case TYPE_GENRES:
-                            title = Util.getMediaGenre(mContext, media);
+                            title = MediaUtils.getMediaGenre(mContext, media);
                             subTitle = null;
                             key = null;
                             break;
@@ -184,9 +185,13 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
                             key = null;
                             break;
                         case TYPE_SONGS:
+                            title = media.getTitle();
+                            subTitle = MediaUtils.getMediaArtist(mContext, media);
+                            key = media.getAlbum() + media.getArtist() + media.getLocation();
+                            break;
                         default:
                             title = media.getTitle();
-                            subTitle = Util.getMediaArtist(mContext, media);
+                            subTitle = MediaUtils.getMediaArtist(mContext, media);
                             key = media.getLocation();
                     }
                     add(title, subTitle, media, key);
@@ -329,14 +334,14 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
             holder = new ViewHolder();
             holder.binding = DataBindingUtil.inflate(inflater, R.layout.audio_browser_item, parent, false);
             v = holder.binding.getRoot();
-            Util.setAlignModeByPref(mAlignMode, (TextView) v.findViewById(R.id.title));
+            UiTools.setAlignModeByPref(mAlignMode, (TextView) v.findViewById(R.id.title));
             holder.viewType = VIEW_MEDIA;
             v.setTag(holder);
         }
 
-        holder.position = position;
         ListItem item = getItem(position);
         holder.binding.setVariable(BR.item, item);
+        holder.binding.setVariable(BR.position, position);
 
         final ArrayList<MediaWrapper> mediaList = mItems.get(position).mMediaList;
         boolean asyncLoad = true;
@@ -392,7 +397,6 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
     }
 
     static class ViewHolder {
-        int position;
         int viewType;
         ViewDataBinding binding;
     }
@@ -573,7 +577,7 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
     @Override
     public void onMoreClick(View v) {
         if (mContextPopupMenuListener != null)
-            mContextPopupMenuListener.onPopupMenu(v, ((ViewHolder) ((LinearLayout)v.getParent().getParent()).getTag()).position);
+            mContextPopupMenuListener.onPopupMenu(v, ((Integer)v.getTag()).intValue());
     }
 
     private static class AudioCoverFetcher extends AsyncImageLoader.CoverFetcher {
