@@ -20,7 +20,6 @@
 
 package org.videolan.vlc.gui.video;
 
-import android.annotation.TargetApi;
 import android.app.Activity;
 import android.content.BroadcastReceiver;
 import android.content.Context;
@@ -28,58 +27,52 @@ import android.content.Intent;
 import android.content.IntentFilter;
 import android.content.res.Configuration;
 import android.content.res.Resources;
-import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
 import android.preference.PreferenceManager;
 import android.support.design.widget.Snackbar;
 import android.support.v4.app.FragmentActivity;
+import android.support.v4.content.LocalBroadcastManager;
 import android.support.v4.util.ArrayMap;
+import android.support.v7.widget.RecyclerView;
 import android.util.DisplayMetrics;
 import android.util.Log;
 import android.view.ContextMenu;
-import android.view.ContextMenu.ContextMenuInfo;
 import android.view.LayoutInflater;
 import android.view.Menu;
 import android.view.MenuInflater;
 import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
-import android.widget.AbsListView;
-import android.widget.AdapterView;
-import android.widget.AdapterView.AdapterContextMenuInfo;
-import android.widget.GridView;
 import android.widget.LinearLayout;
-import android.widget.PopupMenu;
-import android.widget.PopupMenu.OnMenuItemClickListener;
 import android.widget.TextView;
 
 import org.videolan.libvlc.Media;
-import org.videolan.libvlc.util.AndroidUtil;
-import org.videolan.vlc.MediaDatabase;
-import org.videolan.vlc.MediaGroup;
-import org.videolan.vlc.MediaLibrary;
-import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
-import org.videolan.vlc.Thumbnailer;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
+import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
+import org.videolan.vlc.gui.view.SwipeRefreshLayout;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.interfaces.IVideoBrowser;
-import org.videolan.vlc.util.AndroidDevices;
-import org.videolan.vlc.util.Util;
+import org.videolan.vlc.media.MediaDatabase;
+import org.videolan.vlc.media.MediaGroup;
+import org.videolan.vlc.media.MediaLibrary;
+import org.videolan.vlc.media.MediaUtils;
+import org.videolan.vlc.media.MediaWrapper;
+import org.videolan.vlc.media.Thumbnailer;
+import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.WeakHandler;
-import org.videolan.vlc.widget.SwipeRefreshLayout;
+import org.videolan.vlc.view.AutoFitRecyclerView;
 
-import java.io.File;
 import java.util.ArrayList;
 import java.util.List;
 
-public class VideoGridFragment extends MediaBrowserFragment implements ISortable, IVideoBrowser, SwipeRefreshLayout.OnRefreshListener, AdapterView.OnItemClickListener {
+public class VideoGridFragment extends MediaBrowserFragment implements ISortable, IVideoBrowser, SwipeRefreshLayout.OnRefreshListener {
 
     public final static String TAG = "VLC/VideoListFragment";
 
@@ -89,7 +82,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     private static final int DELETE_DURATION = 3000;
 
     protected LinearLayout mLayoutFlipperLoading;
-    protected GridView mGridView;
+    protected AutoFitRecyclerView mGridView;
     protected TextView mTextViewNomedia;
     protected View mViewNomedia;
     protected String mGroup;
@@ -100,9 +93,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
     private MainActivity mMainActivity;
 
-    // Gridview position saved in onPause()
-    private int mGVFirstVisiblePos;
-
     /* All subclasses of Fragment must include a public empty constructor. */
     public VideoGridFragment() { }
 
@@ -129,27 +119,31 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         mLayoutFlipperLoading = (LinearLayout) v.findViewById(R.id.layout_flipper_loading);
         mTextViewNomedia = (TextView) v.findViewById(R.id.textview_nomedia);
         mViewNomedia = v.findViewById(android.R.id.empty);
-        mGridView = (GridView) v.findViewById(android.R.id.list);
+        mGridView = (AutoFitRecyclerView) v.findViewById(android.R.id.list);
         mSwipeRefreshLayout = (SwipeRefreshLayout) v.findViewById(R.id.swipeLayout);
 
         mSwipeRefreshLayout.setColorSchemeResources(R.color.orange700);
         mSwipeRefreshLayout.setOnRefreshListener(this);
 
-        mGridView.setOnScrollListener(new AbsListView.OnScrollListener() {
-            @Override
-            public void onScrollStateChanged(AbsListView view, int scrollState) {
-            }
-
-            @Override
-            public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
-                mSwipeRefreshLayout.setEnabled(firstVisibleItem == 0);
-            }
-        });
+        mGridView.addOnScrollListener(mScrollListener);
         mGridView.setAdapter(mVideoAdapter);
-        mGridView.setOnItemClickListener(this);
         return v;
     }
 
+    RecyclerView.OnScrollListener mScrollListener = new RecyclerView.OnScrollListener() {
+        @Override
+        public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
+            super.onScrollStateChanged(recyclerView, newState);
+        }
+
+        @Override
+        public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
+            int topRowVerticalPosition =
+                    (recyclerView == null || recyclerView.getChildCount() == 0) ? 0 : recyclerView.getChildAt(0).getTop();
+            mSwipeRefreshLayout.setEnabled(topRowVerticalPosition >= 0);
+        }
+    };
+
     @Override
     public void onViewCreated(View view, Bundle savedInstanceState) {
         super.onViewCreated(view, savedInstanceState);
@@ -157,11 +151,11 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
         // init the information for the scan (2/2)
         IntentFilter filter = new IntentFilter();
-        filter.addAction(Util.ACTION_SCAN_START);
-        filter.addAction(Util.ACTION_SCAN_STOP);
-        getActivity().registerReceiver(messageReceiverVideoListFragment, filter);
+        filter.addAction(MediaUtils.ACTION_SCAN_START);
+        filter.addAction(MediaUtils.ACTION_SCAN_STOP);
+        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(messageReceiverVideoListFragment, filter);
         if (mMediaLibrary.isWorking()) {
-            Util.actionScanStart();
+            MediaUtils.actionScanStart();
         }
 
         mAnimator = new VideoGridAnimator(mGridView);
@@ -170,7 +164,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     @Override
     public void onPause() {
         super.onPause();
-        mGVFirstVisiblePos = mGridView.getFirstVisiblePosition();
         mMediaLibrary.setBrowser(null);
         mMediaLibrary.removeUpdateHandler(mHandler);
 
@@ -196,7 +189,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         //Get & set times
         ArrayMap<String, Long> times = MediaDatabase.getInstance().getVideoTimes();
         mVideoAdapter.setTimes(times);
-        mGridView.setSelection(mGVFirstVisiblePos);
         updateViewMode();
         if (mGroup == null && refresh)
             mAnimator.animate();
@@ -214,7 +206,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
     @Override
     public void onDestroyView() {
-        getActivity().unregisterReceiver(messageReceiverVideoListFragment);
+        LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(messageReceiverVideoListFragment);
         super.onDestroyView();
     }
 
@@ -250,35 +242,18 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         // Select between grid or list
         if (!listMode) {
             sidePadding = (int) ((float)outMetrics.widthPixels / 100f * (float)Math.pow(outMetrics.density, 3) / 2f);
-            mGridView.setNumColumns(GridView.AUTO_FIT);
+            mGridView.setNumColumns(-1);
             mGridView.setColumnWidth(res.getDimensionPixelSize(R.dimen.grid_card_width));
-            mGridView.setVerticalSpacing(res.getDimensionPixelSize(R.dimen.grid_card_vertical_spacing));
-            mVideoAdapter.setListMode(false);
         } else {
             sidePadding = res.getDimensionPixelSize(R.dimen.listview_side_padding);
             mGridView.setNumColumns(1);
-            mGridView.setVerticalSpacing(0);
-            mGridView.setHorizontalSpacing(0);
-            mVideoAdapter.setListMode(true);
         }
+        mVideoAdapter.setListMode(listMode);
         sidePadding = Math.max(0, Math.min(100, sidePadding));
         mGridView.setPadding(sidePadding, mGridView.getPaddingTop(),
                 sidePadding, mGridView.getPaddingBottom());
     }
 
-    @Override
-    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
-        MediaWrapper media = mVideoAdapter.getItem(position);
-        if (media == null)
-            return;
-        if (media instanceof MediaGroup) {
-            MainActivity activity = (MainActivity)getActivity();
-            activity.showSecondaryFragment(SecondaryActivity.VIDEO_GROUP_LIST, media.getTitle());
-        }
-        else
-            playVideo(media, false);
-    }
-
     protected void playVideo(MediaWrapper media, boolean fromStart) {
         media.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
         VideoPlayerActivity.start(getActivity(), media.getUri(), fromStart);
@@ -292,7 +267,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     }
 
     private boolean handleContextItemSelected(MenuItem menu, int position) {
-        if (position >= mVideoAdapter.getCount())
+        if (position >= mVideoAdapter.getItemCount())
             return false;
         MediaWrapper media = mVideoAdapter.getItem(position);
         if (media == null)
@@ -327,9 +302,9 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     }
 
     @Override
-    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
+    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
         // Do not show the menu of media group.
-        AdapterContextMenuInfo info = (AdapterContextMenuInfo)menuInfo;
+        ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView.RecyclerContextMenuInfo)menuInfo;
         MediaWrapper media = mVideoAdapter.getItem(info.position);
         if (media == null || media instanceof MediaGroup)
             return;
@@ -350,41 +325,18 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             hasInfo = true;
         media.release();
         menu.findItem(R.id.video_list_info).setVisible(hasInfo);
-        menu.findItem(R.id.video_list_delete).setVisible(!AndroidUtil.isLolliPopOrLater() ||
-                mediaWrapper.getLocation().startsWith("file://" + AndroidDevices.EXTERNAL_PUBLIC_DIRECTORY));
+        menu.findItem(R.id.video_list_delete).setVisible(
+                FileUtils.canWrite(mediaWrapper.getLocation()));
     }
 
     @Override
     public boolean onContextItemSelected(MenuItem menu) {
-        AdapterContextMenuInfo info = (AdapterContextMenuInfo) menu.getMenuInfo();
+        ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView.RecyclerContextMenuInfo) menu.getMenuInfo();
         if (info != null && handleContextItemSelected(menu, info.position))
             return true;
         return super.onContextItemSelected(menu);
     }
 
-    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
-    public void onContextPopupMenu(View anchor, final int position) {
-        if (!AndroidUtil.isHoneycombOrLater()) {
-            // Call the "classic" context menu
-            anchor.performLongClick();
-            return;
-        }
-
-        PopupMenu popupMenu = new PopupMenu(getActivity(), anchor);
-        popupMenu.getMenuInflater().inflate(R.menu.video_list, popupMenu.getMenu());
-        MediaWrapper media = mVideoAdapter.getItem(position);
-        if (media == null)
-            return;
-        setContextMenuItems(popupMenu.getMenu(), media);
-        popupMenu.setOnMenuItemClickListener(new OnMenuItemClickListener() {
-            @Override
-            public boolean onMenuItemClick(MenuItem item) {
-                return handleContextItemSelected(item, position);
-            }
-        });
-        popupMenu.show();
-    }
-
     /**
      * Handle changes on the list
      */
@@ -393,6 +345,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     public void updateItem(MediaWrapper item) {
         mVideoAdapter.update(item);
     }
+
     private void focusHelper(boolean idIsEmpty) {
         View parent = getView();
         if (getActivity() == null || !(getActivity() instanceof MainActivity))
@@ -437,27 +390,8 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                         }
                     }
 
-                    if (getActivity() != null) {
-                        getActivity().runOnUiThread(new Runnable() {
-                            private void addAllV7() {
-                                for (MediaWrapper item : displayList)
-                                    mVideoAdapter.add(item);
-                            }
-
-                            @TargetApi(android.os.Build.VERSION_CODES.HONEYCOMB)
-                            private void addAllV11() {
-                                mVideoAdapter.addAll(displayList);
-                            }
-                            @Override
-                            public void run() {
-                                mVideoAdapter.clear();
-                                if (AndroidUtil.isHoneycombOrLater())
-                                    addAllV11();
-                                else
-                                    addAllV7();
-                            }
-                        });
-                    }
+                    mVideoAdapter.addAll(displayList);
+                    mVideoAdapter.sort();
                     if (mReadyToDisplay)
                         display();
                 }
@@ -514,10 +448,10 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         public void onReceive(Context context, Intent intent) {
             String action = intent.getAction();
 
-            if (action.equalsIgnoreCase(Util.ACTION_SCAN_START)) {
+            if (action.equalsIgnoreCase(MediaUtils.ACTION_SCAN_START)) {
                 mLayoutFlipperLoading.setVisibility(View.VISIBLE);
                 mTextViewNomedia.setVisibility(View.INVISIBLE);
-            } else if (action.equalsIgnoreCase(Util.ACTION_SCAN_STOP)) {
+            } else if (action.equalsIgnoreCase(MediaUtils.ACTION_SCAN_STOP)) {
                 mLayoutFlipperLoading.setVisibility(View.INVISIBLE);
                 mTextViewNomedia.setVisibility(View.VISIBLE);
             }
@@ -540,11 +474,9 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             getActivity().runOnUiThread(new Runnable() {
                 @Override
                 public void run() {
-                    mViewNomedia.setVisibility(mVideoAdapter.getCount() > 0 ? View.GONE : View.VISIBLE);
+                    mVideoAdapter.notifyDataSetChanged();
+                    mViewNomedia.setVisibility(mVideoAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
                     mReadyToDisplay = true;
-                    mVideoAdapter.sort();
-                    mGVFirstVisiblePos = mGridView.getFirstVisiblePosition();
-                    mGridView.setSelection(mGVFirstVisiblePos);
                     mGridView.requestFocus();
                     focusHelper(false);
                 }
@@ -557,12 +489,10 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
     public void deleteMedia(int position){
         final MediaWrapper media = mVideoAdapter.getItem(position);
+        if (media == null)
+            return;
         final String path = media.getUri().getPath();
-        VLCApplication.runBackground(new Runnable() {
-            public void run() {
-                Util.recursiveDelete(VLCApplication.getAppContext(), new File(path));
-            }
-        });
+        FileUtils.asyncRecursiveDelete(path);
         mMediaLibrary.getMediaItems().remove(media);
         mVideoAdapter.remove(media);
         if (mService != null) {
