@@ -25,60 +25,101 @@ import android.databinding.DataBindingUtil;
 import android.databinding.ViewDataBinding;
 import android.graphics.Bitmap;
 import android.graphics.drawable.BitmapDrawable;
+import android.support.annotation.MainThread;
+import android.support.annotation.Nullable;
 import android.support.v4.util.ArrayMap;
-import android.support.v7.widget.CardView;
+import android.support.v7.widget.RecyclerView;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
-import android.widget.ArrayAdapter;
 import android.widget.ImageView;
 
 import org.videolan.vlc.BR;
-import org.videolan.vlc.MediaGroup;
-import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.gui.AsyncImageLoader;
-import org.videolan.vlc.util.BitmapCache;
-import org.videolan.vlc.util.BitmapUtil;
+import org.videolan.vlc.gui.MainActivity;
+import org.videolan.vlc.gui.SecondaryActivity;
+import org.videolan.vlc.gui.helpers.AsyncImageLoader;
+import org.videolan.vlc.gui.helpers.BitmapCache;
+import org.videolan.vlc.gui.helpers.BitmapUtil;
+import org.videolan.vlc.media.MediaGroup;
+import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.Strings;
 
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.Comparator;
 import java.util.Locale;
 
-public class VideoListAdapter extends ArrayAdapter<MediaWrapper>
-                                 implements Comparator<MediaWrapper> {
+public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.ViewHolder>
+        implements Comparator<MediaWrapper>, View.OnLongClickListener {
+
+    public final static String TAG = "VLC/VideoListAdapter";
 
     public final static int SORT_BY_TITLE = 0;
     public final static int SORT_BY_LENGTH = 1;
+
+    public final static int TYPE_LIST = 0;
+    public final static int TYPE_GRID = 1;
+
     public final static int SORT_BY_DATE = 2;
     private int mSortDirection = 1;
     private int mSortBy = SORT_BY_TITLE;
     private boolean mListMode = false;
     private VideoGridFragment mFragment;
+    private volatile ArrayList<MediaWrapper> mVideos = new ArrayList<>();
 
     public static final BitmapDrawable DEFAULT_COVER = new BitmapDrawable(VLCApplication.getAppResources(), BitmapCache.getFromResource(VLCApplication.getAppResources(), R.drawable.ic_cone_o));
 
     public VideoListAdapter(VideoGridFragment fragment) {
-        super(fragment.getActivity(), 0);
+        super();
         mFragment = fragment;
     }
 
-    public final static String TAG = "VLC/MediaLibraryAdapter";
-
-    public synchronized void update(MediaWrapper item) {
-        int position = getPosition(item);
-        if (position != -1) {
-            remove(item);
-            insert(item, position);
-        }
+    @Override
+    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
+        boolean listMode = viewType == TYPE_LIST;
+        LayoutInflater inflater = (LayoutInflater) parent.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
+        View v = inflater.inflate(mListMode ? R.layout.video_list_card : R.layout.video_grid_card, parent, false);
+        return new ViewHolder(v, listMode);
     }
 
+    @Override
+    public void onBindViewHolder(ViewHolder holder, int position) {
+        MediaWrapper media = mVideos.get(position);
+        boolean asyncLoad = true;
+
+        holder.binding.setVariable(BR.scaleType, ImageView.ScaleType.CENTER);
+        final Bitmap bitmap = BitmapUtil.getPictureFromCache(media);
+        if (bitmap != null) {
+            if (bitmap.getWidth() != 1 && bitmap.getHeight() != 1) {
+                asyncLoad = false;
+                holder.binding.setVariable(BR.scaleType, ImageView.ScaleType.FIT_CENTER);
+                holder.binding.setVariable(BR.cover, new BitmapDrawable(VLCApplication.getAppResources(), bitmap));
+            } else
+                holder.binding.setVariable(BR.cover, DEFAULT_COVER);
+        } else {
+            holder.binding.setVariable(BR.cover, DEFAULT_COVER);
+        }
+
+        fillView(holder, media);
+
+        holder.binding.setVariable(BR.position, position);
+        holder.binding.setVariable(BR.media, media);
+        holder.binding.setVariable(BR.handler, mClickHandler);
+        holder.binding.executePendingBindings();
+        holder.itemView.setOnLongClickListener(this);
+        if (asyncLoad)
+            AsyncImageLoader.LoadImage(new VideoCoverFetcher(holder.binding, media), null);
+    }
+
+    @MainThread
     public void setTimes(ArrayMap<String, Long> times) {
         boolean notify = false;
         // update times
-        for (int i = 0; i < getCount(); ++i) {
-            MediaWrapper media = getItem(i);
+        for (int i = 0; i < getItemCount(); ++i) {
+            MediaWrapper media = mVideos.get(i);
             Long time = times.get(media.getLocation());
             if (time != null) {
                 media.setTime(time);
@@ -133,10 +174,15 @@ public class VideoListAdapter extends ArrayAdapter<MediaWrapper>
     public void sort() {
         if (!isEmpty())
             try {
-                super.sort(this);
+                Collections.sort(mVideos, this);
             } catch (ArrayIndexOutOfBoundsException e) {} //Exception happening on Android 2.x
     }
 
+    public boolean isEmpty()
+    {
+        return mVideos.isEmpty();
+    }
+
     @Override
     public int compare(MediaWrapper item1, MediaWrapper item2) {
         int compare = 0;
@@ -155,63 +201,54 @@ public class VideoListAdapter extends ArrayAdapter<MediaWrapper>
         return mSortDirection * compare;
     }
 
-    /**
-     * Display the view of a file browser item.
-     */
-    @Override
-    public View getView(final int position, View convertView, ViewGroup parent) {
-        ViewHolder holder;
-        View v = convertView;
-
-        if (v == null || (((ViewHolder)v.getTag(R.layout.video_grid)).listmode != mListMode)) {
-            LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
-
-            holder = new ViewHolder();
-            holder.binding = DataBindingUtil.inflate(inflater, mListMode ? R.layout.video_list_card : R.layout.video_grid_card, parent, false);
-            v = holder.binding.getRoot();
-            holder.listmode = mListMode;
-            v.setTag(R.layout.video_grid, holder);
-        } else {
-            holder = (ViewHolder) v.getTag(R.layout.video_grid);
-        }
-
-        if (position >= getCount() || position < 0)
-            return v;
-
-        holder.position = position;
-
-        MediaWrapper media = getItem(position);
-        boolean asyncLoad = true;
-
-        holder.binding.setVariable(BR.scaleType, ImageView.ScaleType.CENTER);
-        final Bitmap bitmap = BitmapUtil.getPictureFromCache(media);
-        if (bitmap != null) {
-            if (bitmap.getWidth() != 1 && bitmap.getHeight() != 1) {
-                asyncLoad = false;
-                holder.binding.setVariable(BR.scaleType, ImageView.ScaleType.FIT_CENTER);
-                holder.binding.setVariable(BR.cover, new BitmapDrawable(VLCApplication.getAppResources(), bitmap));
-            } else
-                holder.binding.setVariable(BR.cover, DEFAULT_COVER);
-        } else {
-            holder.binding.setVariable(BR.cover, DEFAULT_COVER);
-        }
-
-        fillView(holder, media);
-
-        holder.binding.setVariable(BR.media, media);
-        holder.binding.setVariable(BR.handler, mClickHandler);
-        holder.binding.executePendingBindings();
-        if (asyncLoad)
-            AsyncImageLoader.LoadImage(new VideoCoverFetcher(holder.binding, media), null);
-        return v;
+    @Nullable
+    public MediaWrapper getItem(int position) {
+        if (position < 0 || position > mVideos.size())
+            return null;
+        else
+            return mVideos.get(position);
     }
 
-    public ClickHandler mClickHandler = new ClickHandler();
-    public class ClickHandler {
-        public void onMoreClick(View v){
-            if (mFragment != null)
-                    mFragment.onContextPopupMenu(v, ((ViewHolder) ((CardView)v.getParent().getParent()).getTag(R.layout.video_grid)).position);
+    public void add(MediaWrapper item) {
+        mVideos.add(item);
+    }
+
+    @MainThread
+    public void remove(MediaWrapper item) {
+        int position = getItemPosition(item);
+        if (position == -1)
+            return;
+        mVideos.remove(position);
+        notifyItemRemoved(position);
+    }
+
+    private int getItemPosition(MediaWrapper mw) {
+        if (mw == null || mVideos.isEmpty())
+            return -1;
+        for (int i = 0 ; i < mVideos.size(); ++i){
+            if (mw.equals(mVideos.get(i)))
+                return i;
         }
+        return -1;
+    }
+
+    public void addAll(Collection<MediaWrapper> items) {
+        mVideos.clear();
+        mVideos.addAll(items);
+
+    }
+
+    @MainThread
+    public void update(MediaWrapper item) {
+        int position = mVideos.indexOf(item);
+        if (position != -1) {
+            mVideos.set(position, item);
+        }
+        notifyItemChanged(position);
+    }
+
+    public void clear() {
+        mVideos.clear();
     }
 
     private void fillView(ViewHolder holder, MediaWrapper media) {
@@ -225,7 +262,7 @@ public class VideoListAdapter extends ArrayAdapter<MediaWrapper>
             group = true;
             MediaGroup mediaGroup = (MediaGroup) media;
             int size = mediaGroup.size();
-            resolution = getContext().getResources().getQuantityString(R.plurals.videos_quantity, size, size);
+            resolution = VLCApplication.getAppResources().getQuantityString(R.plurals.videos_quantity, size, size);
         } else {
             group = false;
             /* Time / Duration */
@@ -252,12 +289,6 @@ public class VideoListAdapter extends ArrayAdapter<MediaWrapper>
         holder.binding.setVariable(BR.progress, progress);
     }
 
-    static class ViewHolder {
-        boolean listmode;
-        int position;
-        ViewDataBinding binding;
-    }
-
     public void setListMode(boolean value) {
         mListMode = value;
     }
@@ -287,4 +318,58 @@ public class VideoListAdapter extends ArrayAdapter<MediaWrapper>
             }
         }
     }
+
+    @Override
+    public long getItemId(int position) {
+        return 0l;
+    }
+
+    @Override
+    public int getItemCount() {
+        return mVideos.size();
+    }
+
+    @Override
+    public int getItemViewType(int position) {
+        return super.getItemViewType(position);
+    }
+
+    static class ViewHolder extends RecyclerView.ViewHolder {
+        boolean listmode;
+        ViewDataBinding binding;
+
+        public ViewHolder(View itemView, boolean listMode) {
+            super(itemView);
+            binding = DataBindingUtil.bind(itemView);
+            this.listmode = listMode;
+        }
+    }
+
+    public ClickHandler mClickHandler = new ClickHandler();
+    public class ClickHandler {
+        public void onClick(View v){
+            MediaWrapper media = mVideos.get(((Integer) v.findViewById(R.id.item_more).getTag()).intValue());
+            if (media instanceof MediaGroup) {
+                MainActivity activity = (MainActivity) mFragment.getActivity();
+                activity.showSecondaryFragment(SecondaryActivity.VIDEO_GROUP_LIST, media.getTitle());
+            } else {
+                media.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+                VideoPlayerActivity.start(v.getContext(), media.getUri(), media.getTitle());
+            }
+        }
+        public void onMoreClick(View v){
+            if (mFragment == null)
+                return;
+            mFragment.mGridView.openContextMenu(((Integer) v.getTag()).intValue());
+        }
+    }
+
+    @Override
+    public boolean onLongClick(View v) {
+        if (mFragment == null)
+            return false;
+        int position = ((Integer) v.findViewById(R.id.item_more).getTag()).intValue();
+        mFragment.mGridView.openContextMenu(position);
+        return true;
+    }
 }
