@@ -31,7 +31,7 @@ import org.videolan.libvlc.MediaPlayer;
 import org.videolan.libvlc.util.VLCUtil;
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.HWDecoderUtil;
-import org.videolan.vlc.MediaWrapper;
+import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 
@@ -50,8 +50,8 @@ public class VLCOptions {
     public static final int HW_ACCELERATION_DECODING = 1;
     public static final int HW_ACCELERATION_FULL = 2;
 
-    public static ArrayList<String> getLibOptions(Context context) {
-        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
+    public static ArrayList<String> getLibOptions() {
+        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
 
         ArrayList<String> options = new ArrayList<String>(50);
 
@@ -59,7 +59,7 @@ public class VLCOptions {
         final boolean timeStreching = pref.getBoolean("enable_time_stretching_audio", timeStrechingDefault);
         final String subtitlesEncoding = pref.getString("subtitle_text_encoding", "");
         final boolean frameSkip = pref.getBoolean("enable_frame_skip", false);
-        String chroma = pref.getString("chroma_format", null);
+        String chroma = pref.getString("chroma_format", VLCApplication.getAppResources().getString(R.string.chroma_format_default));
         if (chroma != null)
             chroma = chroma.equals("YV12") && !AndroidUtil.isGingerbreadOrLater() ? "" : chroma;
         final boolean verboseMode = pref.getBoolean("enable_verbose_mode", true);
@@ -91,6 +91,8 @@ public class VLCOptions {
             options.add("--network-caching=" + networkCaching);
         options.add("--androidwindow-chroma");
         options.add(chroma != null ? chroma : "RV32");
+        options.add("--audio-resampler");
+        options.add(getResampler());
 
         options.add(verboseMode ? "-vvv" : "-vv");
         return options;
@@ -136,6 +138,11 @@ public class VLCOptions {
         return ret;
     }
 
+    private static String getResampler() {
+        final VLCUtil.MachineSpecs m = VLCUtil.getMachineSpecs();
+        return m.processors > 2 ? "soxr" : "ugly";
+    }
+
     public static void setMediaOptions(Media media, Context context, int flags) {
         boolean noHardwareAcceleration = (flags & MediaWrapper.MEDIA_NO_HWACCEL) != 0;
         boolean noVideo = (flags & MediaWrapper.MEDIA_VIDEO) == 0;
